from wayback import WaybackClient
from archivenow import archivenow

from alexandria.shell import print_color, suppress_stdout
from alexandria.shell.color import colors

from anchorage.anchor_utils.aesthetic import str_log_info, str_log_error, str_log_success


def add(url, archive='ia', api_key=None, overwrite=False):
    """
    Archive a website in one of the four archives supported by Archive Now (archivenow).

    TODO: Recognize internetarchive upload error messages as failures.

    :param url: URL of website to be archived.
    :param archive: List or string specifying archives to which to save the website.
                     Available archives:
                        - 'all': All archives
                        - 'ia': Internet Archive (default)
                        - 'is': Archive.is
                        - 'mg': Megalodon.jp
                        - 'cc': Perma.cc
    :param api_key: Perma.cc API key. Format:
                        {"cc_api_key":"$YOUR-Perma-cc-API-KEY"}
    :param overwrite: Archive URL even if it's already present in the Internet Archive.
    """

    def upload(url):
        try:
            if archive == 'cc':
                with suppress_stdout():
                    archive_url = archivenow.push(url, archive, api_key)[0]
            else:
                with suppress_stdout():
                    archive_url = archivenow.push(url, archive)[0]
            log = str_log_success(url + " -> -> -> " + archive_url)
            return log
        except BaseException as e:
            print(str_log_error(url))
            print_color(e, "red")

    try:
        archive_latest = next(WaybackClient().search(url))   # Search for URL using the WaybackMachine API
        if overwrite:
            return upload(url)
        else:
            return str_log_info("SKIPPED", url + " => => => " + archive_latest[7])
    except:
        # If bookmark search yields "0" error (defined by Python _wayback_)
        return upload(url)



from pyfiglet import Figlet

from alexandria.data_structs.string import join_set_distance
from alexandria.shell import print_color, str_log, str_color


def str_log_info(kind, msg):
    return str_log(kind, msg,
                   msg_color="", msg_bg_color="",
                   )


def str_log_success(msg):
    return str_log("SUCCESS", msg,
                   msg_color="brightCyan", msg_bg_color="",
                   kind_color="brightGreen", kind_bg_color=""
                   )


def str_log_error(msg):
    return str_log("FAILURE", msg,
                   msg_color="red", msg_bg_color="",
                   kind_color="brightRed", kind_bg_color=""
                   )


def str_log_progress(fraction, t_elapsed, t_remaining,
                     c_fr="brightCyan", c_bg_fr="",
                     c_te="", c_bg_te="",
                     c_tr="brightCyan", c_bg_tr="",
                     l=11, m=15, n=20):
    # Times
    def s_to_hms(_n):
        hours, remainder = divmod(_n, 3600)
        minutes, seconds = divmod(remainder, 60)
        return f'{int(hours)}:{int(minutes)}:{seconds:.2f}'

    t_elapsed = s_to_hms(t_elapsed)
    t_remaining = "to go -> " + s_to_hms(t_remaining)

    # Set distances
    r = join_set_distance(fraction, "::", l)
    r = join_set_distance(r, t_elapsed, m)
    r = join_set_distance(r, " ::", n)
    r = join_set_distance(r, t_remaining, len(r)+2)

    str_t, str_kind, str_msg = r.split(" :: ")

    # Take care not to color whitespace for KIND string
    #     It is assumed that all possible whitespace has
    #     been added by <<join_set_distance>> and is thus
    #     placed at the end of the string.
    def countstrip(s):
        n0 = len(s)
        s = s.rstrip()
        n1 = len(str_kind)
        n_ws = n0 - n1
        return s, n_ws
    str_fraction, n_ws_fr = countstrip(fraction)
    str_te, n_ws_te = countstrip(t_elapsed)
    str_tr, n_ws_tr = countstrip(t_remaining)

    # Color
    c_str_fr    = str_color(str_t, c_fr, c_bg_fr) + " " * n_ws_fr
    c_str_te = str_color(str_kind, c_te, c_bg_te) + " " * n_ws_te
    c_str_tr  = str_color(str_msg, c_tr, c_bg_tr) + " " * n_ws_tr

    # Join in final string
    r = " :: ".join([c_str_fr, c_str_te, c_str_tr])
    return r


def newline():
    """
    Print an empty line. Purely for aesthetic reasons.
    """
    print("")


def title(text="Anchorage", font="big", color="cyan"):
    """
    Generates the Anchorage CLI title.

    :param text: Text to be rendered
    :param font: Figlet font to render the title
    :param color: Title color
    """
    f = Figlet(font=font)
    print_color(f.renderText(text), color)


class colors:
    """
    ANSI color codes for use with `tqdm` to generate colored progress bars.
    """

    reset = "\033[0m"

    # Black
    fgBlack = "\033[30m"
    fgBrightBlack = "\033[30;1m"
    bgBlack = "\033[40m"
    bgBrightBlack = "\033[40;1m"

    # Red
    fgRed = "\033[31m"
    fgBrightRed = "\033[31;1m"
    bgRed = "\033[41m"
    bgBrightRed = "\033[41;1m"

    # Green
    fgGreen = "\033[32m"
    fgBrightGreen = "\033[32;1m"
    bgGreen = "\033[42m"
    bgBrightGreen = "\033[42;1m"

    # Yellow
    fgYellow = "\033[33m"
    fgBrightYellow = "\033[33;1m"
    bgYellow = "\033[43m"
    bgBrightYellow = "\033[43;1m"

    # Blue
    fgBlue = "\033[34m"
    fgBrightBlue = "\033[34;1m"
    bgBlue = "\033[44m"
    bgBrightBlue = "\033[44;1m"

    # Magenta
    fgMagenta = "\033[35m"
    fgBrightMagenta = "\033[35;1m"
    bgMagenta = "\033[45m"
    bgBrightMagenta = "\033[45;1m"

    # Cyan
    fgCyan = "\033[36m"
    fgBrightCyan = "\033[36;1m"
    bgCyan = "\033[46m"
    bgBrightCyan = "\033[46;1m"

    # White
    fgWhite = "\033[37m"
    fgBrightWhite = "\033[37;1m"
    bgWhite = "\033[47m"
    bgBrightWhite = "\033[47;1m"



#!/usr/run/env python37

from __future__ import print_function, unicode_literals

import sys

from alexandria.shell import print_color

from anchorage.bookmarks import bookmarks, path, load
from anchorage.anchor import anchor_online, anchor_locally
from anchorage.anchor_infrs.infrastructure import init, check_install, read_config
from anchorage.anchor_utils.shell import shell
from anchorage.anchor_utils.system import operating_system
from anchorage.anchor_utils.aesthetic import newline, title


try:
    from PyInquirer import style_from_dict, Token, prompt
except ImportError:
    print("PyInquirer import error: prompt-toolkits version != 1.0.14")
    print("Installing prompt-toolkit==1.0.14")
    shell("pip install prompt-toolkit==1.0.14")
    sys.exit("Success! Please run again")


def main():
    """
    # Anchorage library CLI.

    ## 1. Dependency check
        - Choice: overwrite config.toml if found
    ## 2. Browser choice
    ## 3. Bookmark filter
        - Local files
        - String match
            - Directories
            - Bookmark name
            - Bookmark URL
        - Substring match
            - Directories
            - Bookmark name
            - Bookmark URL
        - Regex match
            - Directories
            - Bookmark name
            - Bookmark URL
        - *Duplicate URLs are excluded by default*
    ## 4. Archive choice
        - Online
            - Skip sites already saved in TIA, or generate current snapshots for all sites in the collection
        - Local
            - Local archive directory input
    ## 5. Confirmation
    """

    # Title
    print('\nWelcome to the')
    title("Anchorage  cli")
    print("                                        Antonio Lopez Rivera, 2021\n")

    # CLI interface look
    style = style_from_dict({
        Token.QuestionMark:     '#E91E63 bold',
        Token.Selected:         '#673AB7 bold',
        Token.Instruction:      '',                # default
        Token.Answer:           '#9cdeff bold',
        Token.Question:         '',
    })

    """
    Dialogue
    """
    # 1. Dependency check
    dp_check = [{
                    'type': 'confirm',
                    'name': 'ready',
                    'message': 'Are you ready to proceed?',
                    'default': True
                }]

    if not prompt(dp_check, style=style)['ready']:
        print_color("\n     ~Operation cancelled\n", "red")
        sys.exit()

    print_color("\n     ~Running dependency checks", "white")
    shell("pip3 install --upgrade anchorage")

    if check_install() == 0:
        print("")
        reset_config = [{
            'type': 'confirm',
            'name': 'reset',
            'message': f'Previous config detected: reset config.toml?',
            'default': False,
        }]
        reset = prompt(reset_config, style=style)['reset']
        newline()
    else:
        reset = False

    init(reset)

    print_color("     ~Everything in order!", "green")
    print_color("     ~Edit configuration in ~/.anchorage/config.toml\n", "green")

    # 2. Browser choice
    cfg = read_config()
    brow_choice = [{
                    'type': 'list',
                    'name': 'browser',
                    'message': f'Please choose the browser from which to archive all bookmarks.',
                    'choices': list(map(lambda s: s.title(), list(cfg.keys()))),
                    'filter': lambda n: n.lower()
                   }]
    browser = prompt(brow_choice, style=style)['browser']
    newline()
    if cfg[browser][operating_system()] == "?":
        print_color("     ~The path to this browser hasn't been set up yet in your configuration file.", "red")
        print(f"     ~Edit ~/anchorage/config.toml to include the path for your {browser.title()} "
              f"bookmarks file and try again.\n")
        sys.exit()
    try:
        bmk_dict = load(path(browser))
    except FileNotFoundError:
        print_color("     ~Error: file not found.", "red")
        print_color("\n     ~Edit ~/anchorage/config.toml to include the correct bookmark file path "
                          "for your browser and try again.\n", "red")
        sys.exit()

    # 3. Bookmark filter
    #     - Local files
    #     - By string match
    #     - By substring match
    #     - By regex match
    bk_filt = [{
                    'type': 'checkbox',
                    'name': 'drop',
                    'message': f'If desired, specify which elements to exclude from the archiving process.',
                    'choices': [
                        {
                            'name': 'Local files'
                        },
                        {
                            'name': 'Match string'
                        },
                        {
                            'name': 'Match substring'
                        },
                        {
                            'name': 'Regex'
                        },
                    ],
                    'filter': lambda lst: [n.lower() if isinstance(n, str) else n for n in lst]
                  }]

    filter_kind = lambda kind: [{
                                    'type': 'checkbox',
                                    'name': 'kind',
                                    'message': f'Specify targets for {kind} filter.',
                                    'choices': [
                                        {
                                            'name': 'Directories'
                                        },
                                        {
                                            'name': 'Names'
                                        },
                                        {
                                            'name': 'URLs'
                                        },
                                    ],
                                    'filter': lambda lst: [n.lower() if isinstance(n, str) else n for n in lst]
                                }]

    filter_input = lambda terms: prompt([{
                                            'type': 'input',
                                            'name': 'input',
                                            'message': f'Enter a comma-separated list of {terms} to filter out.',
                                            'default': '',
                                            'filter': lambda string: string.split(",")
                                        }], style=style)['input']

    filter_regex = lambda terms: prompt([{
                                            'type': 'input',
                                            'name': 'input',
                                            'message': f'Enter regex formula to filter out {terms}.',
                                            'default': ''
                                         }], style=style)['input']

    drop_list = prompt(bk_filt, style=style)['drop']
    newline()

    drop_dirs       = None
    drop_names      = None
    drop_urls       = None
    drop_dir_subs   = None
    drop_name_subs  = None
    drop_url_subs   = None
    drop_dir_regex  = None
    drop_name_regex = None
    drop_url_regex  = None

    if 'match string' in drop_list:
        kind = prompt(filter_kind('string'), style=style)['kind']
        newline()
        if 'directories' in kind:
            drop_dirs = filter_input('bookmark directory names')
            newline()
        if 'names' in kind:
            drop_names = filter_input('bookmark names')
            newline()
        if 'urls' in kind:
            drop_urls = filter_input('bookmark URLs')
            newline()
    if 'match substring' in drop_list:
        kind = prompt(filter_kind('substring'), style=style)['kind']
        newline()
        if 'directories' in kind:
            drop_dir_subs = filter_input('bookmark directory name substrings')
            newline()
        if 'names' in kind:
            drop_name_subs = filter_input('bookmark name substrings')
            newline()
        if 'urls' in kind:
            drop_url_subs = filter_input('bookmark URL substrings')
            newline()
    if 'regex' in drop_list:
        kind = prompt(filter_kind('regex'), style=style)['kind']
        newline()
        if 'directories' in kind:
            drop_dir_regex = filter_regex('bookmark directory names')
            newline()
        if 'names' in kind:
            drop_name_regex = filter_regex('bookmark names')
            newline()
        if 'urls' in kind:
            drop_url_regex = filter_regex('bookmark URLs')
            newline()

    print_color("     ~Applying filters to bookmark collection", "white")

    bmk = bookmarks(bmk_dict,
                    drop_local_files='local files' in drop_list,
                    drop_dirs=drop_dirs,
                    drop_names=drop_names,
                    drop_urls=drop_urls,
                    drop_dirs_subs=drop_dir_subs,
                    drop_names_subs=drop_name_subs,
                    drop_urls_subs=drop_url_subs,
                    drop_dirs_regex=drop_dir_regex,
                    drop_names_regex=drop_name_regex,
                    drop_urls_regex=drop_url_regex,
                    )
    n_bookmarks = len(bmk.bookmarks)
    n_directories = bmk.n_dirs
    print_color(f"     ~Done! Found {n_bookmarks} bookmarks in {n_directories} directories.\n", "green")

    # 4. Archive choice
    archive_choice = [{
                    'type': 'list',
                    'name': 'archive',
                    'message': f'Choose an archiving method.',
                    'choices': ['Online', 'Local'],
                    'filter': lambda n: n.split(' ')[0].lower()
                     }]

    archive = prompt(archive_choice, style=style)['archive']
    newline()

    if archive == 'online':         # Online archive: save existing entries?
        print_color(f"     ~By default websites will not be archived if a previous image "
                          f"exists in The Internet Archive.\n", "white")
        ovw_online = [{
                          'type': 'confirm',
                          'name': 'overwrite',
                          'message': 'Save snapshots even if previous ones exist?',
                          'default': False
                     }]
        overwrite = prompt(ovw_online, style=style)['overwrite']
        newline()
        if overwrite:
            print_color(f"     ~Current snapshots will be saved for your entire collection. "
                              f"Expect a runtime of {n_bookmarks*15/3600:.2f} to {n_bookmarks*60/3600:.2f} "
                              f"hours.\n", "green")
        else:
            print_color(f"     ~Expect a runtime of {n_bookmarks*1.5/3600:.2f} to "
                              f"{n_bookmarks*3/3600:.2f} hours.\n", "green")
    if archive == 'local':          # Local archive directory
        dir_prompt = [{
                          'type': 'input',
                          'name': 'dir',
                          'message': 'Enter the relative or full path of the archive directory.',
                          'default': './anchor'
                     }]
        archive_dir = prompt(dir_prompt, style=style)['dir']
        newline()

    # 5. Log level
    log_prompt = [{
                        'type': 'list',
                        'name': 'loglevel',
                        'message': 'Enter the relative or full path of the archive directory.',
                        'choices': ['  - Full log output', '  - Progress bar', '  - Suppress all output'],
                        'filter': lambda choice: {'Full log output'    : 0,
                                                  'Progress bar'       : 20,
                                                  'Suppress all output': 50}[choice]
                  }]
    loglevel = prompt(log_prompt, style=style)['loglevel']
    newline()

    # 6. Confirmation
    confirmation = lambda br, nb: [{
                                        'type': 'confirm',
                                        'name': 'go',
                                        'message': 'All set! Proceed to archive your'
                                                   + " " + str(br) + " " + str(nb) + " " +
                                                   'bookmark collection?',
                                        'default': True
                                   }]
    conf_prompt = confirmation(browser.title(), n_bookmarks)

    go = prompt(conf_prompt, style=style)['go']
    newline()

    if go:
        if archive == 'online':                     # Anchor online
            anchor_online(bmk,
                          overwrite=overwrite,
                          loglevel=loglevel)
        elif archive == 'local':                    # Anchor locally
            anchor_locally(bmk,
                           archive=archive_dir,
                           loglevel=loglevel)
    else:
        print_color("     ~Operation cancelled\n", "red")



import codecs

config = codecs.decode(r'# Welcome to the                                                       \n'
                       r'#     /\              | |                                      | (_)   \n'
                       r'#    /  \   _ __   ___| |__   ___  _ __ __ _  __ _  ___     ___| |_|   \n'
                       r'#   / /\ \ |  _ \ / __|  _ \ / _ \|  __/ _  |/ _  |/ _ \   / __| | |   \n'
                       r'#  / ____ \| | | | (__| | | | (_) | | | (_| | (_| |  __/  | (__| | |   \n'
                       r'# /_/    \_\_| |_|\___|_| |_|\___/|_|  \__,_|\__, |\___|   \___|_|_|   \n'
                       r'#                                            __/ |                     \n'
                       r'#                                           |___/                      \n'
                       r'#                                                 Configuration file   \n'
                       r'#\n'
                       r'# Below you can see the path to the Bookmarks JSON/JSONLZ4 file for a  \n'
                       r'# series of browsers.\n'
                       r'# \n'
                       r'# Set your default browser to one of the available to skip browser     \n'
                       r'# choice when using the Anchorage CLI.                                 \n'
                       r'# \n'
                       r'# You can add new browsers and the path to their bookmarks file. You   \n'
                       r'# will then be able to set the new browser as the default, or choose   \n'
                       r'# it in the Anchorage CLI.                                             \n'
                       r'# \n'
                       r'# If using a custom browser **it must comply with the Chromium or      \n'
                       r'# Firefox bookmark storage conventions (i.e. dictionary key naming)**. \n'
                       r'# If this is not the case, export your bookmarks to a supported browser\n'
                       r'# and proceed.                                                         \n'
                       r'# \n'
                       r'# Path convention:                                                     \n'
                       r'#      Windows:            APPDATA/<your path>                         \n'
                       r'#      Linux, MacOS:       <full path>                                 \n'
                       r'# \n'
                       r'#                                           Antonio Lopez Rivera, 2021 \n'
                       r'\n'
                       r'[chrome]\n'
                       r'linux = "?"\n'
                       r'macos = "?"\n'
                       r'windows = "Local/Google/Chrome/User Data/Default/Bookmarks"\n'
                       r'\n'
                       r'[firefox]\n'
                       r'linux = "?"\n'
                       r'macos = "?"\n'
                       r'windows = "Roaming/Mozilla/Firefox/Profiles/mfeew61f.default-release/bookmarkbackups/"\n'
                       r'\n'
                       r'[edge]\n'
                       r'linux = "?"\n'
                       r'macos = "?"\n'
                       r'windows = "Local/Microsoft/Edge/User Data/Default/Bookmarks"\n'
                       r'\n'
                       r'["edge beta"]\n'
                       r'linux = "?"\n'
                       r'macos = "?"\n'
                       r'windows = "Local/Microsoft/Edge Beta/User Data/Default/Bookmarks"\n'
                       ,
                       'unicode_escape')




import unittest

from anchorage.anchor_tools.online import add


class OnlineArchiveTests(unittest.TestCase):

    def test_add(self):
        print("\nONLINE ARCHIVE TESTS: ADD NEW SITE\n")
        print(add("https://m.youtube.com/watch?v=XhExfYBfypI", "ia", overwrite=True)[0])

    def test_existing(self):
        print("\nONLINE ARCHIVE TESTS: AVOID SAVING IF ALREADY ARCHIVED\n")
        assert add("http://paulgraham.com/think.html", "ia") == "Bookmark already present in the Internet Archive"
